package org.bdware.sc.registry;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import org.bdware.sc.engine.JSONTool;
import wrp.jdk.nashorn.api.scripting.ScriptObjectMirror;
import java.sql.SQLException;
import org.bdware.sc.util.JsonUtil;

import java.sql.Timestamp;
import java.util.HashMap;
import java.util.Map;

import static org.bdware.sc.engine.JSONTool.convertMirrorToJson;


public class RegistryDB {
  private static int stateCode;

  public static void init(ScriptObjectMirror arg, String requester) {
    try {
      JsonObject jsonData = convertMirrorToJson(arg).getAsJsonObject();
      DBConnector.url = jsonData.get("DBUrl").getAsString();
      DBConnector.password = jsonData.get("password").getAsString();
      DBConnector.userName = jsonData.get("username").getAsString();
      DBConnector.init();
      DBConnector.connect();

      Map<String, Object> item = new HashMap<>();
      item.put("pubkey", requester);
      item.put("type", "admin");
      item.put("username", jsonData.get("adminName").getAsString());
      DBConnector.insert("registry", "user", item);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public static Object getRegistryInfo(ScriptObjectMirror scriptMirrorObject) {
    JsonObject jo = convertMirrorToJson(scriptMirrorObject).getAsJsonObject();
    JsonObject ret = new JsonObject();
    String info = DBConnector.queryInfoByMetaID(jo.get("doid").getAsString());
    if(!info.equals("")) {
      ret = JsonUtil.parseStringAsJsonObject(info);
      stateCode = 0;
    } else {
      stateCode = -1;
    }
    return JSONTool.convertJsonElementToMirror(ret);
  }

  public static Object querySampleByMetaID(ScriptObjectMirror scriptMirrorObject) {
    JsonObject jo = convertMirrorToJson(scriptMirrorObject).getAsJsonObject();
    JsonObject ret = new JsonObject();
    String sample = DBConnector.querySampleByMetaID(jo.get("localMetaID").getAsString());
    if(!sample.equals("")) {
      ret = JsonUtil.parseStringAsJsonObject(sample);
      stateCode = 0;
    } else {
      stateCode = -1;
    }
    return JSONTool.convertJsonElementToMirror(ret);
  }

  public static Object queryMetaStandardsByTimeAndOffset(ScriptObjectMirror scriptMirrorObject) {
    JsonObject jo = convertMirrorToJson(scriptMirrorObject).getAsJsonObject();
    Timestamp startTime = new Timestamp(jo.get("createStartDate").getAsLong());
    Timestamp endTime = new Timestamp(jo.get("createEndDate").getAsLong());
    int offset = jo.get("offset").getAsInt();
    int count = jo.get("count").getAsInt();
    JsonObject ret = DBConnector.queryMetaStandardByDate(startTime, endTime, offset, count);
    if(ret == null) {
      stateCode = -1;
    } else {
      stateCode = 0;
      return JSONTool.convertJsonElementToMirror(ret);
    }
    return new JsonObject();
  }

  public static Object queryMapRuleByOffset(ScriptObjectMirror scriptMirrorObject) {
    JsonObject jo = convertMirrorToJson(scriptMirrorObject).getAsJsonObject();
    int offset = jo.get("offset").getAsInt();
    int count = jo.get("count").getAsInt();
    String localMetaStandardDoid = jo.get("localMetaStandardDoid").getAsString();
    JsonObject ret = DBConnector.queryMapRuleByOffset(offset, count, localMetaStandardDoid);
    if(ret == null) {
      stateCode = -1;
    } else {
      stateCode = 0;
      return JSONTool.convertJsonElementToMirror(ret);
    }
    return new JsonObject();
  }

  public static int getLastExecuteStatus() {
    return stateCode;
  }

}
