package org.bdware.datanet.searchengine;


import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import org.bdware.sc.engine.JSONTool;
import org.junit.Test;
import org.neo4j.graphdb.*;
import org.neo4j.graphdb.factory.GraphDatabaseFactory;
import org.neo4j.graphdb.index.Index;
import org.neo4j.graphdb.traversal.Evaluators;
import org.neo4j.graphdb.traversal.TraversalDescription;

import java.io.File;
import java.util.Iterator;
import java.util.Map;

public class MetaGraph {

    //    private static final String DB_PATH = "testgraph.db";

    private static String DB_PATH="./MetaGraph.db";
//    private static enum LABEL {
//        DO, REAL, STANDARD
//    }

    private GraphDatabaseService graphDB;
    public MetaGraph(String name){
        DB_PATH="./"+name+".db";
        graphDB = new GraphDatabaseFactory().newEmbeddedDatabase(new File(DB_PATH));
    }
    //DO类型


    /*
    {
        "doid":"xxx",
        "maskinfo":{
            "stu_no":"@int(5,10)",
            "name":"cname"
        }
    }
     */
    public void createDoNode(String doid,JsonObject jo) {
        Transaction tx = graphDB.beginTx();
        Node Do = graphDB.createNode(new NodeLabel("DO"));//命名规则
        Do.setProperty("doid",doid);
        Iterator iter = jo.entrySet().iterator();
        while (iter.hasNext()) {
            Map.Entry entry = (Map.Entry) iter.next();
            Do.setProperty(entry.getKey().toString(),entry.getValue().toString());
        }
        tx.success();
        tx.close();
        //配置do的mask
    }
    public void iterDO() {

        Transaction tx = graphDB.beginTx();
        ResourceIterator<Node> iterator = graphDB.findNodes(new NodeLabel("DO"));
        for (ResourceIterator<Node> it = iterator; it.hasNext(); ) {
            Node node = it.next();
            String doid=(String)node.getProperty("doid");
            String id=(String)node.getProperty("id");
            System.out.println(doid + " " + id+ " ");
        }

        tx.success();
        tx.close();
    }
    //param:{"doid1":"xxx","doid2":"yyy","key1":"studentid","key2":"studentid"}
    public void createRelationDo2Do(JsonObject jo){
        Transaction tx = graphDB.beginTx();
        String doid1=jo.get("id1").getAsString();
        String doid2=jo.get("id2").getAsString();
        String key1=jo.get("key1").getAsString();
        String key2=jo.get("key2").getAsString();

        Node node1=graphDB.findNode(new NodeLabel("DO"),"doid",doid1);
        Node node2=graphDB.findNode(new NodeLabel("DO"),"doid",doid2);
        System.out.println(jo);

        node1.createRelationshipTo(node2, new RelationType(jo.toString()));
        tx.success();
        tx.close();
    }



    //真实标签类型
    public void createRealNode(String key,String value) {
        Transaction tx = graphDB.beginTx();
        Node realField = graphDB.createNode(new NodeLabel("realField"));//命名规则
        realField.setProperty("key",key);
        realField.setProperty("value",value);
        tx.success();
        tx.close();

    }

    //标准标签

    public void createStandardNode(String key,String value) {
        Transaction tx = graphDB.beginTx();
        Node standardField = graphDB.createNode(new NodeLabel("standardField"));//命名规则
        standardField.setProperty("key",key);
        standardField.setProperty("value",value);
        tx.success();
        tx.close();
    }
    public void iterRealField() {

        Transaction tx = graphDB.beginTx();
        ResourceIterator<Node> iterator = graphDB.findNodes(new NodeLabel("DO"));
        for (ResourceIterator<Node> it = iterator; it.hasNext(); ) {
            Node node = it.next();
            String doid=(String)node.getProperty("doid");
            String id=(String)node.getProperty("id");
            System.out.println(doid + " " + id+ " ");
        }

        tx.success();
        tx.close();
    }

    public void iterStandardField() {

        Transaction tx = graphDB.beginTx();
        ResourceIterator<Node> iterator = graphDB.findNodes(new NodeLabel("DO"));
        for (ResourceIterator<Node> it = iterator; it.hasNext(); ) {
            Node node = it.next();
            String doid=(String)node.getProperty("doid");
            String id=(String)node.getProperty("id");
            System.out.println(doid + " " + id+ " ");
        }

        tx.success();
        tx.close();
    }
//    public void init() {
//        graphDB = new GraphDatabaseFactory().newEmbeddedDatabase(new File(DB_PATH));
//    }

//    /**创建数据*/
//    public void create() {
//        Transaction tx = graphDB.beginTx();
//
//        Node student = graphDB.createNode(new CaseLabel("STUDENT"));//命名规则
//        student.setProperty("name", "@student");//数据类型命名规则
//        student.setProperty("id", "@int(2001210000,2001210999)");
//
//        Node course = graphDB.createNode(new CaseLabel("COURSE"));//命名规则
//        course.setProperty("name", "[枚举or读文件]");//数据类型命名规则
//        course.setProperty("id", "@int(10000,10999)");
//
//        student.createRelationshipTo(course, new RelTypes("CRIME"));
//
//
//
//        Node case1 = graphDB.createNode(new CaseLabel("CASEINFO"));
//        case1.setProperty("name", "案件1");
//        case1.setProperty("address", "南山");
//
//        Node case2 = graphDB.createNode(new CaseLabel("CASEINFO"));
//        case2.setProperty("name", "案件2");
//        case2.setProperty("address", "福田");
//
//        Node case3 = graphDB.createNode(new CaseLabel("CASEINFO"));
//        case3.setProperty("name", "案件3");
//        case3.setProperty("address", "龙华");
//
//        Node userA = graphDB.createNode(new CaseLabel("PERSON"));
//        userA.setProperty("name", "A");
//        userA.setProperty("idcard", "150302198012228239");
//        userA.setProperty("tel", "13685246639");
//
//        Node userB = graphDB.createNode(new CaseLabel("PERSON"));
//        userB.setProperty("name", "B");
//        userB.setProperty("idcard", "370634199208304929");
//        userB.setProperty("tel", "13885246670");
//
//        Node userC = graphDB.createNode(new CaseLabel("PERSON"));
//        userC.setProperty("name", "C");
//        userC.setProperty("idcard", "430721198802065735");
//        userC.setProperty("tel", "13966704782");
//
//        Node userD = graphDB.createNode(new CaseLabel("PERSON"));
//        userD.setProperty("name", "D");
//        userD.setProperty("idcard", "522730198707118747");
//        userD.setProperty("tel", "13670478962");
//
//        Node userE = graphDB.createNode(new CaseLabel("PERSON"));
//        userE.setProperty("name", "E");
//        userE.setProperty("idcard", "622926198609158032");
//        userE.setProperty("tel", "13047829667");
//
//        Node userF = graphDB.createNode(new CaseLabel("PERSON"));
//        userF.setProperty("name", "F");
//        userF.setProperty("idcard", "500114197706138305");
//        userF.setProperty("tel", "13478296670");
//
//        Node userG = graphDB.createNode(new CaseLabel("PERSON"));
//        userG.setProperty("name", "G");
//        userG.setProperty("idcard", "500114106138305152");
//        userG.setProperty("tel", "13476670156");
//
//        Node userH = graphDB.createNode(new CaseLabel("PERSON"));
//        userH.setProperty("name", "H");
//        userH.setProperty("idcard", "500114197704751236");
//        userH.setProperty("tel", "13296156670");
//
//        case1.createRelationshipTo(userA, new RelTypes("CRIME"));
//        case1.createRelationshipTo(userB, new RelTypes("CRIME"));
//        case1.createRelationshipTo(userC, new RelTypes("CRIME"));
//        case1.createRelationshipTo(userD, new RelTypes("CRIME"));
//
//        case2.createRelationshipTo(userB, new RelTypes("CRIME"));
//        case2.createRelationshipTo(userC, new RelTypes("CRIME"));
//        case2.createRelationshipTo(userE, new RelTypes("CRIME"));
////        case2.createRelationshipTo(userD, RelTypes.CRIME);
//
//
//        case3.createRelationshipTo(userF, new RelTypes("CRIME"));
//        case3.createRelationshipTo(userG, new RelTypes("CRIME"));
//        case3.createRelationshipTo(userH, new RelTypes("CRIME"));
//
//        tx.success();
//        tx.close();
//
//    }
//
//    public void search(String username) {
//        Transaction tx = graphDB.beginTx();
//        System.out.println(graphDB.getAllNodes().stream().findFirst());
//        Node startNode = graphDB.findNode( "name", username);
//
//        Iterable<Relationship> iterable = startNode
//                .getRelationships(new RelTypes("CRIME"), Direction.INCOMING);
//        for (Relationship r : iterable) {
//
//            Node node = r.getStartNode();
//            long id = node.getId();
//            String name = (String)node.getProperty("name");
////            String idcard = (String)node.getProperty("idcard");
////            String tel = (String)node.getProperty("tel");
//
//            System.out.println(id + " " + name+ " ");
//        }
//
//        tx.success();
//        tx.close();
//    }
//
//    /**遍历查找**/
//    public void searchUser(String username) {
//        Transaction tx = graphDB.beginTx();
//
//        Node startNode = graphDB.findNode(new CaseLabel("PERSON"), "name", username);
////        Node startNode = graphDB.findNode(new CaseLabel("PERSON"), "name", "B");
//
//        TraversalDescription td = graphDB.traversalDescription();
//        td.relationships(new RelTypes("CRIME"), Direction.OUTGOING);//沿着关系 TONG
//        td.depthFirst();//设置深度优先
////        td.evaluator(Evaluators.excludeStartPosition());
//
//        //从开始节点开始
//
//        Iterable<Node> it = td.traverse(startNode).nodes();
//        for (Node node : it) {
//            long id = node.getId();
//            if( node.hasLabel( new CaseLabel("PERSON") )) {
//                String name = (String)node.getProperty("name");
//                String idcard = (String)node.getProperty("idcard");
//                String tel = (String)node.getProperty("tel");
//
//                System.out.println(id + " " + name+ " " + idcard + " " + tel);
//            } else {
////                String name = (String)node.getProperty("name");
////                String address = (String)node.getProperty("address");
////                System.out.println(id + " " + name+ " " + address);
//            }
//        }
//
//        tx.success();
//        tx.close();
//    }
//


//
//    /**查询所有作案人员信息*/
//    public void searchAllPerson() {
//
//
//
//        Transaction tx = graphDB.beginTx();
//
//        ResourceIterator<Node> iterator = graphDB.findNodes(new CaseLabel("PERSON"));
//        for (ResourceIterator<Node> it = iterator; it.hasNext(); ) {
//            Node node = it.next();
//            long id = node.getId();
//            String name = (String)node.getProperty("name");
//            String idcard = (String)node.getProperty("idcard");
//            String tel = (String)node.getProperty("tel");
//
//            System.out.println(id + " " + name+ " " + idcard + " " + tel);
//        }
//
//        tx.success();
//        tx.close();
//    }
}

class RelationType implements RelationshipType {
    private String name;
    public RelationType(String name) {
        this.name = name;
    }

    public String name(){
        return name;
    }
}
class NodeLabel implements Label {
    private String name;
    public NodeLabel(String name) {
        this.name = name;
    }

    public String name(){
        return name;
    }
}
