package org.bdware.datanet.searchengine;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.*;
import org.apache.lucene.index.*;
import org.apache.lucene.queryparser.classic.MultiFieldQueryParser;
import org.apache.lucene.queryparser.classic.ParseException;
import org.apache.lucene.queryparser.classic.QueryParser;
import org.apache.lucene.search.*;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.util.BytesRef;
import org.apache.lucene.util.Version;
import org.bdware.sc.boundry.utils.RocksDBUtil;
import org.rocksdb.RocksIterator;
import org.wltea.analyzer.lucene.IKAnalyzer;
import java.io.*;
import java.nio.file.Paths;
import java.util.*;
public class MetaIndex {
    private final Logger LOGGER = LogManager.getLogger(MetaIndex.class);
    public  FSDirectory indexDir;
    public  boolean isEmpty = false;//描述当前repo的文件是否存在
    private IndexWriter indexWriter;
    private static final int COUNT=20;
    private static final int OFFSET=0;
    MetaIndex(String name){
        initIndex(name);
    }
    MetaIndex(){
        initIndex("SearchEngineIndex"+System.currentTimeMillis());
    }

    public void initIndex(String Dirname)  {
        try {
            File dir = new File(Dirname);
            if (!dir.exists()) {
                LOGGER.info("make metaIndex dir ");
                dir.mkdirs();
                isEmpty = true;
            }
            indexDir = FSDirectory.open(Paths.get(dir.toURI()));
            Analyzer analyzer = new IKAnalyzer();
            indexWriter = new IndexWriter(indexDir, new IndexWriterConfig(analyzer));
            indexWriter.commit();


        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    public void indexRepo(DoMeta meta) throws IOException {
        try {
            //IndexWriter indexWriter = new IndexWriter(indexDir, new IndexWriterConfig());
            Document document = new Document();
            // System.out.println(meta.metadata);
            for (Map.Entry entry : meta.metadata.entrySet()) {
                String key=entry.getKey().toString();
                System.out.println(entry.getValue());
                String val=entry.getValue().toString().substring(1,entry.getValue().toString().length()-1);
                System.out.println(key+"    "+val);
                //if(key.equals("doid")||key.equals("owner"))
                //    document.add(new StringField(key,val, Field.Store.YES));
                //范围类型
                if(key.equals("createTime")||key.equals("createDate")){
                    document.add(new StringField(key,val,Field.Store.YES));
                }

                else if(key.equals("doId")){
                    document.add(new StringField(key,val,Field.Store.YES));
                    //document.add(new LongPoint(key,Long.valueOf(val)),Field.Store.YES);
                }

                else{
                    document.add(new TextField(key,val, Field.Store.YES));
                }

                //System.out.println("eee"+entry.getKey().toString()+" "+entry.getValue().toString().substring(1,entry.getValue().toString().length()-1));
            }
            indexWriter.addDocument(document);
            indexWriter.commit();
            isEmpty = false;
        }
        catch (Exception e) {
            e.printStackTrace();
        }

    }

    public void updateRepoList(List<DoMeta> DoRepos)
            throws IOException, ParseException {
        IndexReader indexReader = null;
        for (DoMeta r : DoRepos) {
            String doId=r.metadata.get("doId").toString();
            if(isEmpty){
                indexRepo(r);
                LOGGER.info("--> index when dir is not empty");
                LOGGER.info("contract " + doId + " --> actually to index");
            }
            else{
                try {
                    indexReader = DirectoryReader.open(indexDir);
                    IndexSearcher indexSearcher = new IndexSearcher(indexReader);
                    LOGGER.debug( doId+ "--> try to index");
                    //Query query = new TermQuery(new Term("doid", "AIBD/doip.Repo1001"));
                    Query query = new TermQuery(new Term("doId", doId));
                    TopDocs docs = indexSearcher.search(query, 10);


                    LOGGER.debug(docs.scoreDocs);

                    if(docs.scoreDocs == null || docs.scoreDocs.length == 0) {
                        indexRepo(r);
                        LOGGER.info("--> index when dir is not empty");
                        LOGGER.info("contract " + doId + " --> actually to index");
                    }
                    docs = indexSearcher.search(query, 10);
                } catch (Exception e) {
                    LOGGER.warn("getting index failed! " + e.getMessage());
                }
            }

        }
        if (null != indexReader) {
            indexReader.close();
        }
    }
    public void updateRepo(DoMeta DoRepo)
            throws IOException, ParseException {
        IndexReader indexReader = null;

            String doId=DoRepo.metadata.get("doId").getAsString();
            if(isEmpty){
                indexRepo(DoRepo);
                LOGGER.info("--> index when dir is not empty");
                LOGGER.info("contract " + doId + " --> actually to index");
            }
            else{
                try {
                    indexReader = DirectoryReader.open(indexDir);
                    IndexSearcher indexSearcher = new IndexSearcher(indexReader);
                    LOGGER.debug( doId+ "--> try to index");
                    Query query = new TermQuery(new Term("doId", doId));
                    TopDocs docs = indexSearcher.search(query, 10);
                    LOGGER.debug(docs.scoreDocs);
                    System.out.println("bb"+docs.scoreDocs.length);
                    if(docs.scoreDocs == null || docs.scoreDocs.length == 0) {
                        indexRepo(DoRepo);
                        LOGGER.info("--> index when dir is not empty");
                        LOGGER.info("--> index when dir is not empty");
                        LOGGER.info("contract " + doId + " --> actually to index");
                    }
                    docs = indexSearcher.search(query, 10);
                } catch (Exception e) {
                    LOGGER.warn("getting index failed! " + e.getMessage());
                }
            }


        if (null != indexReader) {
            indexReader.close();
        }
    }
    public void deleteRepo(String doId) {

        try {
            IndexWriter indexWriter = new IndexWriter(indexDir, new IndexWriterConfig());
            Query query=new TermQuery( new Term("doId",doId));
            System.out.println("query"+query.toString());
            indexWriter.deleteDocuments(query);
            indexWriter.commit();
            indexWriter.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    //模糊搜索
    public  String search(JsonObject jo) {
        try {
            if (!jo.has("type")) {
                return "missing arguments";
            }
            String type=jo.get("type").getAsString();
            if (!jo.has("keyword")) return "missing arguments: keyword";
            int offset=OFFSET;
            if (jo.has("offset")) offset= jo.get("offset").getAsInt();
            offset= Math.max(offset, 0);
            int count=COUNT;
            if (jo.has("count")) count= jo.get("count").getAsInt();
            count=count<0?COUNT:count;
            String keyword = jo.get("keyword").getAsString();
            System.out.println(type+"  "+keyword);
//            Analyzer analyzer = new IKAnalyzer();
//            QueryParser queryParser = new QueryParser(type, analyzer);
//            Query rmQuery = queryParser.parse(keyword);

            DirectoryReader indexReader = DirectoryReader.open(indexDir);
            IndexSearcher indexSearcher = new IndexSearcher(indexReader);
            Query query = new TermQuery(new Term(type, keyword));
            TopDocs docs = indexSearcher.search(query, 10);
            System.out.println("length   "+docs.scoreDocs.length);
            ResultModel resultModel = null;
            if (docs.scoreDocs != null && docs.scoreDocs.length > 0)
                resultModel = paginate(docs, indexReader,count,offset);

            if (resultModel != null) {
                return resultModel.toString();
            }
            ResultModel noDataResult=new ResultModel();
            return noDataResult.toString();
        }
        catch (Exception e) {
            ByteArrayOutputStream bo = new ByteArrayOutputStream();
            e.printStackTrace(new PrintStream(bo));
            return bo.toString();
        }
    }
    //按照范围查询
    public  String searchByRange(JsonObject jo) {
        try {
            if (!jo.has("type")) {
                return "missing arguments";
            }
            //if (!jo.has("keyword")) return "missing arguments: keyword";
            if (!jo.has("from")) return "missing arguments: from";
            if (!jo.has("to")) return "missing arguments: to";
            String type = jo.get("type").getAsString();
            //String keyword = jo.get("keyword").getAsString();
            String from = jo.get("from").getAsString();
            String to = jo.get("to").getAsString();

            int offset = OFFSET;
            if (jo.has("offset")) offset = jo.get("offset").getAsInt();
            offset = Math.max(offset, 0);
            int count = COUNT;
            if (jo.has("count")) count = jo.get("count").getAsInt();
            count = count < 0 ? COUNT : count;
//            Analyzer analyzer = new IKAnalyzer();
//            QueryParser queryParser = new QueryParser(type, analyzer);
//            Query rmQuery = queryParser.parse(keyword);

            DirectoryReader indexReader = DirectoryReader.open(indexDir);
            IndexSearcher indexSearcher = new IndexSearcher(indexReader);
            Query query = new TermRangeQuery(type, new BytesRef(from), new BytesRef(to), true, true);
            //Query query = new TermQuery(new Term(type, keyword));
            TopDocs docs = indexSearcher.search(query, 10);

            ResultModel resultModel = null;
            if (docs.scoreDocs != null && docs.scoreDocs.length > 0) {
                resultModel = paginate(docs, indexReader, count,offset);

                if (resultModel != null) {
                    return resultModel.toString();
                }
                ResultModel noDataResult = new ResultModel();
                return noDataResult.toString();
            }
        } catch (Exception e) {
            ByteArrayOutputStream bo = new ByteArrayOutputStream();
            e.printStackTrace(new PrintStream(bo));
            return bo.toString();
        }
        ResultModel noDataResult = new ResultModel();
        return noDataResult.toString();
    }
    /*
        {"params":[{"type":"doid","ketword":"DBRepo"},{"type":"name","ketword":"xxx"}],"count":10,"offset":0}
         */
    public String searchByMultiTag(JsonObject jo) {
        try {
            int offset = OFFSET;
            if (jo.has("offset")) offset = jo.get("offset").getAsInt();
            offset = Math.max(offset, 0);
            int count = COUNT;
            if (jo.has("count")) count = jo.get("count").getAsInt();
            count = count < 0 ? COUNT : count;

            //System.out.println(count+"  "+offset);
            JsonArray parmas=jo.get("params").getAsJsonArray();
            parmas.get(0).getAsJsonObject().get("keyword");
            String[] type=new String[parmas.size()];
            String[] keyword=new String[parmas.size()];
            BooleanClause.Occur[] flags = new BooleanClause.Occur[parmas.size()];
            for(int i=0;i<parmas.size();i++){
                type[i]=parmas.get(i).getAsJsonObject().get("type").getAsString();
                keyword[i]=parmas.get(i).getAsJsonObject().get("keyword").getAsString();
                flags[i]=BooleanClause.Occur.SHOULD;
                //System.out.println(type[i]+"   "+keyword[i]);
            }

            DirectoryReader indexReader = DirectoryReader.open(indexDir);
            IndexSearcher indexSearcher = new IndexSearcher(indexReader);
            Query query = MultiFieldQueryParser.parse(keyword, type, flags, new IKAnalyzer());
            TopDocs docs = indexSearcher.search(query, 10);
            ResultModel resultModel = null;
            if (docs.scoreDocs != null && docs.scoreDocs.length > 0) {
                resultModel = paginate(docs, indexReader, count,offset);
                if (resultModel != null) {
                    return resultModel.toString();
                }
                ResultModel noDataResult = new ResultModel();
                return noDataResult.toString();
            }

        }catch (Exception e){
            ByteArrayOutputStream bo = new ByteArrayOutputStream();
            e.printStackTrace(new PrintStream(bo));
            return bo.toString();
        }
        ResultModel noDataResult = new ResultModel();
        return noDataResult.toString();
    }

    public  ResultModel paginate(TopDocs docs, IndexReader reader,int count,int offset ) throws IOException {
        ResultModel resultModel = new ResultModel();
        resultModel.setContractCount(docs.totalHits.value);
        ScoreDoc[] scoreDocs = docs.scoreDocs;
        List<DoMeta> doMetaList = new ArrayList<>();

        if (scoreDocs != null) {
            for (int i = offset; i < scoreDocs.length && i < offset+count; i++) {
                Document document = reader.document(scoreDocs[i].doc);
                JsonObject jo=new JsonObject();
                for (IndexableField in : document.getFields()) {
                    String key=in.name();
                    String val=in.stringValue();
                    jo.addProperty(key,val);
                }
                DoMeta repometa=new DoMeta(jo);
                doMetaList.add(repometa);
            }
        }
        resultModel.setDoMetaList(doMetaList);
        return resultModel;
    }


}
