import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.analysis.tokenattributes.CharTermAttribute;
import org.apache.lucene.analysis.tokenattributes.OffsetAttribute;
import org.apache.lucene.analysis.tokenattributes.PositionIncrementAttribute;
import org.apache.lucene.analysis.tokenattributes.TypeAttribute;
import org.junit.Test;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
//import org.wltea.analyzer.lucene.IKAnalyzer;

import java.io.StringReader;

public class luceneTest {
    @Test
    public void StandardAnalyzerTest() throws Exception{
        //英文测试
        String text="An IndexWriter creaters and maintains an index.";
        Analyzer analyzer=new StandardAnalyzer();
        displayTokens(analyzer,text);
        //中文测试
        String text1="我们这款产品底层逻辑是打通信息屏障，创建行业生态。顶层设计是聚焦用户感和赛道，通过差异化和颗粒度达到引爆点。交付价值是在垂直领域采用复用大法达成持久收益。抽离透传归因分析作为抓手为产品赋能，体验度量作为闭环的评判标准。亮点是载体，优势是链路。思考整个生命周期，完善逻辑，考虑资源倾斜。方法论是组合拳达到平台化标准。";
        displayTokens(analyzer,text1);
    }
    /**
     58       * IKAnalyzerTest分词法测试,对中文支持很好，词库分词
     59       * @throws Exception
     60       */
    @Test
    public void IKAnalyzerTest() throws Exception{
        //英文测试
        String text="An IndexWriter creaters and maintains an index.";
//        Analyzer analyzer=new IKAnalyzer();
        Analyzer analyzer=new StandardAnalyzer();
        displayTokens(analyzer,text);
        //中文测试
        String text1="我们这款产品底层逻辑是打通信息屏障，创建行业生态。顶层设计是聚焦用户感和赛道，通过差异化和颗粒度达到引爆点。交付价值是在垂直领域采用复用大法达成持久收益。抽离透传归因分析作为抓手为产品赋能，体验度量作为闭环的评判标准。亮点是载体，优势是链路。思考整个生命周期，完善逻辑，考虑资源倾斜。方法论是组合拳达到平台化标准。";
        displayTokens(analyzer,text1);
    }
    public static void displayTokens(Analyzer analyzer, String text) throws Exception {
       System.out.println("当前使用的分词器：" + analyzer.getClass().getName());
       //分词流，即将对象分词后所得的Token在内存中以流的方式存在，也说是说如果在取得Token必须从TokenStream中获取，而分词对象可以是文档文本，也可以是查询文本。
       TokenStream tokenStream = analyzer.tokenStream("content", new StringReader(text));
       //表示token的首字母和尾字母在原文本中的位置。比如I'm的位置信息就是(0,3)，需要注意的是startOffset与endOffset的差值并不一定就是termText.length()，
       //因为可能term已经用stemmer或者其他过滤器处理过；
       OffsetAttribute offsetAttribute = tokenStream.addAttribute(OffsetAttribute.class);
       //这个有点特殊，它表示tokenStream中的当前token与前一个token在实际的原文本中相隔的词语数量，用于短语查询。比如： 在tokenStream中[2:a]的前一个token是[1:I'm ]，
       //它们在原文本中相隔的词语数是1，则token="a"的PositionIncrementAttribute值为1；
       PositionIncrementAttribute positionIncrementAttribute = tokenStream.addAttribute(PositionIncrementAttribute.class);

       CharTermAttribute charTermAttribute= tokenStream.addAttribute(CharTermAttribute.class);

       //表示token词典类别信息，默认为“Word”，比如I'm就属于<APOSTROPHE>，有撇号的类型；
       TypeAttribute typeAttribute = tokenStream.addAttribute(TypeAttribute.class);
       tokenStream.reset();

       int position = 0;
       while (tokenStream.incrementToken()) {
           int increment = positionIncrementAttribute.getPositionIncrement();
           if(increment > 0) {
               position = position + increment;
           }
           int startOffset = offsetAttribute.startOffset();
           int endOffset = offsetAttribute.endOffset();
           String term ="输出结果为："+ charTermAttribute.toString();
           System.out.println("第"+position+"个分词，分词内容是:[" + term + "]" + "，分词内容的开始结束位置为：(" + startOffset + "-->" + endOffset + ")，类型是：" + typeAttribute.type());
       }
       tokenStream.close();
    }
}
